
from werkzeug.exceptions import Forbidden

from odoo import _
from odoo.addons.saas_worker.compat import route, http, request
from odoo.addons.sale_subscription.controllers import portal


class MaintenanceController(http.Controller):
    @route("/my/subscription/<int:subscription_id>/maintenance_activation", auth='user', methods=['POST'], website=True, csrf=True)
    def maintenance_activation(self, subscription_id, **kwargs):
        subscription = request.env['sale.order'].sudo().browse(subscription_id)
        if not subscription.exists() or not subscription._user_is_referrer(user=request.env.user):
            raise Forbidden()

        if subscription._get_maintenance_line():
            message = _("The maintenance subscription could not be activated since it was already activated.")
            message_class = "alert-danger"
        else:
            rate = subscription._get_default_maintenance_rate()
            subscription = subscription._maintenance_activate(rate)
            message = _("The maintenance is activated successfully.")
            message_class = "alert-success"
            if subscription.plan_id.billing_period_unit == 'year':
                action = subscription.prepare_upsell_order()
                upsell_so = request.env['sale.order'].browse(action['res_id'])
                # Remove empty line since it's a bit confusing for portal user but
                # keep maintenance line and prorata period note.
                mnt_line = upsell_so._get_maintenance_line()
                upsell_so.order_line.filtered(
                    lambda l: l.product_uom_qty == 0 and not l.display_type and l != mnt_line
                ).sudo().unlink()
                upsell_so.sudo().state = 'sent'
                return request.redirect("/my/orders/{}?access_token={}".format(
                    upsell_so.id, upsell_so._portal_ensure_token()
                ))


        return request.redirect("/my/subscriptions/{}?access_token={}&message={}&message_class={}".format(
            subscription.id, subscription.access_token, message, message_class))


class HideCloseSubscription(portal.CustomerPortal):
    @route()
    def subscription(self, account_id, access_token=None, message='', message_class='', **kw):
        response = super().subscription(account_id, access_token=access_token, message=message, message_class=message_class, **kw)

        # immediately return reponse if user is trying to print the subscription
        if kw.get('report_type') in ('html', 'pdf', 'text'):
            return response

        sale_order = response.qcontext.get('sale_order')
        if not sale_order:
            return response

        if sale_order._get_maintenance_loc_count() > 0 and sale_order.maintenance_main_subscription_id:
            response.qcontext.update(display_close=False)
        return response
